"""
Module contains different helper functions.
Module is independent from any okta modules.
"""

import re


def to_snake_case(string):
    """
    Converts string to snake case.

    Args:
        string (str): input string in any case

    Returns:
        str: string converted to snake case

    Example:
        >>> to_snake_case('lowerCamelCaseString')
        'lower_camel_case_string'
    """
    return re.sub(r'(?<!^)(?=[A-Z])', '_', string).lower()


def to_lower_camel_case(string):
    """
    Converts string to lower camel case.

    Args:
        string (str): input string in any case

    Returns:
        str: string converted to lower camel case

    Example:
        >>> to_lower_camel_case('snake_case_string')
        'snakeCaseString'
    """
    components = string.split('_')
    # lower first letter in the first component
    if components[0]:
        components[0] = components[0][0].lower() + components[0][1:]
    # join other components with first capitalized first letter
    return components[0] + ''.join(x.title() for x in components[1:])



"""
Copyright 2022 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.models.subscription\
    import Subscription
from okta.utils import format_url
from okta.api_client import APIClient


class SubscriptionClient(APIClient):
    """
    A Client object for the Subscription resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_role_subscriptions(
            self, roleTypeOrRoleId,
            keep_empty_params=False
    ):
        """
        When roleType List all subscriptions of a Role. Else wh
        en roleId List subscriptions of a Custom Role
        Args:
            role_type_or_role_id {str}
        Returns:
            list: Collection of Subscription instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/roles/{roleTypeOrRoleId}/subscriptions
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Subscription)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(Subscription(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_role_subscription_by_notification_type(
            self, roleTypeOrRoleId, notificationType,
            keep_empty_params=False
    ):
        """
        When roleType Get subscriptions of a Role with a specif
        ic notification type. Else when roleId Get subscription
        of a Custom Role with a specific notification type.
        Args:
            role_type_or_role_id {str}
            notification_type {str}
        Returns:
            Subscription
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/roles/{roleTypeOrRoleId}/subscriptions
                /{notificationType}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Subscription)

        if error:
            return (None, response, error)

        try:
            result = Subscription(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def subscribe_role_subscription_by_notification_type(
            self, roleTypeOrRoleId, notificationType,
            keep_empty_params=False
    ):
        """
        When roleType Subscribes a Role to a specific notificat
        ion type. When you change the subscription status of a
        Role, it overrides the subscription of any individual u
        ser of that Role. Else when roleId Subscribes a Custom
        Role to a specific notification type. When you change t
        he subscription status of a Custom Role, it overrides t
        he subscription of any individual user of that Custom R
        ole.
        Args:
            role_type_or_role_id {str}
            notification_type {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/roles/{roleTypeOrRoleId}/subscriptions
                /{notificationType}/subscribe
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def unsubscribe_role_subscription_by_notification_type(
            self, roleTypeOrRoleId, notificationType,
            keep_empty_params=False
    ):
        """
        When roleType Unsubscribes a Role from a specific notif
        ication type. When you change the subscription status o
        f a Role, it overrides the subscription of any individu
        al user of that Role. Else when roleId Unsubscribes a C
        ustom Role from a specific notification type. When you
        change the subscription status of a Custom Role, it ove
        rrides the subscription of any individual user of that
        Custom Role.
        Args:
            role_type_or_role_id {str}
            notification_type {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/roles/{roleTypeOrRoleId}/subscriptions
                /{notificationType}/unsubscribe
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def subscribe_user_subscription_by_notification_type(
            self, userId, notificationType,
            keep_empty_params=False
    ):
        """
        Subscribes a User to a specific notification type. Only
        the current User can subscribe to a specific notificat
        ion type. An AccessDeniedException message is sent if r
        equests are made from other users.
        Args:
            user_id {str}
            notification_type {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/users/{userId}/subscriptions
                /{notificationType}/subscribe
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def unsubscribe_user_subscription_by_notification_type(
            self, userId, notificationType,
            keep_empty_params=False
    ):
        """
        Unsubscribes a User from a specific notification type.
        Only the current User can unsubscribe from a specific n
        otification type. An AccessDeniedException message is s
        ent if requests are made from other users.
        Args:
            user_id {str}
            notification_type {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/users/{userId}/subscriptions
                /{notificationType}/unsubscribe
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)



import json
import xmltodict

from okta.api_client import APIClient
from okta.utils import convert_absolute_url_into_relative_url


class OktaAPIResponse():
    """
    Class for defining the wrapper of an Okta API response.
    Allows for paginated results to be retrieved easily.
    """

    def __init__(self, request_executor, req, res_details, response_body="",
                 data_type=None):
        self._url = res_details.url
        self._headers = req["headers"]
        self._resp_headers = res_details.headers
        self._self = None  # Link to first page of results
        self._body = None  # First page of results
        self._type = data_type
        self._status = res_details.status

        # Status on if there's a next page of results (based on generator)
        self._next = None

        # Request Executor for future calls
        self._request_executor = request_executor

        # Build response body based on content type
        if "application/xml" in res_details.content_type:
            self.build_xml_response(response_body)
        elif "application/json" in res_details.content_type or \
                "" == res_details.content_type:
            self.build_json_response(response_body)
        else:
            # Save response as text
            self._body = response_body

        # Get links for next if more results exist
        if self._body is not None:
            self.extract_pagination(res_details.links)
            self._current = self._body

    def get_headers(self):
        """
        Returns the response body of the Okta API Response.

        Returns:
            CIMultiDictProxy: dict-like object
        """
        return self._resp_headers

    def get_body(self):
        """
        Returns the response body of the Okta API Response.

        Returns:
            dict: Dictionary format of response
        """
        return self._body

    def get_status(self):
        """
        Returns HTTP Status Code of response

        Returns:
            int: HTTP Code
        """
        return self._status

    def get_type(self):
        """
        Returns datatype of the API response

        Returns:
            class: type of object
        """
        return self._type

    def build_json_response(self, response_body):
        """
        Converts JSON response text into Python dictionary.

        Args:
            response_body (str): Response text
        """
        self._body = json.loads(response_body)

    def build_xml_response(self, response_body):
        """
        Converts XML response text into Python dictionary.

        Args:
            response_body ([type]): [description]
        """
        self._body = xmltodict.parse(response_body, xml_attribs=False)

    def extract_pagination(self, links):
        """
        Parses the Link Headers in the Okta API Response to see if there
        are more results.

        Args:
            links (dict): Dictionary object of values in the 'Link' header
        """

        # Check for 'self' link
        if "self" in links:
            self._self = convert_absolute_url_into_relative_url(links["self"]["url"].human_repr())
        # Check for 'next' link
        if "next" in links:
            self._next = convert_absolute_url_into_relative_url(links["next"]["url"].human_repr())

    def has_next(self):
        """
        Returns if there is another page after the last response retrieved
        (Determined by generator).

        Returns:
            bool: Existence of next page of results
        """
        return self._next is not None

    async def next(self):
        """
        Generator iterating function. Retrieves the next page of results
        from the API.

        Returns:
            json: Next page of results
        """
        next_page, error = await self.get_next().__anext__()
        if error:
            return (None, error)
        if self._type is not None:
            result = []
            for item in next_page:
                result.append(
                    self._type(
                        APIClient.form_response_body(item)
                    )
                )
            return (result, None)

        return (next_page, error)

    async def get_next(self):
        """
        Async Generator function for results pagination.

        Yields:
            (json, Exception): Next page of results, Error raised
        """
        while self._next:
            # Retrieve next when next page exists

            # Create and fire request
            next_request, error = await self._request_executor.create_request(
                "GET", self._next, {}, self._headers)
            if error:
                # Return None if error and set next to none
                self._next = None
                yield (None, error)

            req, res_details, resp_body, error = await \
                self._request_executor.fire_request(next_request)
            if error:
                # Return None if error and set next to none
                self._next = None
                yield (None, error)

            if next_request:
                # create new response and update generator values
                next_response = OktaAPIResponse(
                    self._request_executor, req, res_details, resp_body)
                self._next = next_response._next
                # yield next page
                yield (next_response.get_body(), None)



from aenum import MultiValueEnum


class OktaObject:
    """
    Base object for all Okta datatypes
    """

    def __init__(self, config=None):
        pass

    def __repr__(self):
        return str(vars(self))

    def as_dict(self):
        """
        Returns dictionary object of
        {{pascalCase model.modelName}} object.
        """
        result = {}
        for key, val in self.request_format().items():
            if val is None:
                continue
            if isinstance(val, list):
                formatted_list = []
                for item in val:
                    if isinstance(item, OktaObject):
                        formatted_list.append(item.as_dict())
                    else:
                        formatted_list.append(item)
                result[key] = formatted_list
            elif not isinstance(val, OktaObject):
                result[key] = val
            elif issubclass(type(val), MultiValueEnum):
                result[key] = val.value
            else:
                result[key] = val.as_dict()
        return result

    def request_format(self):
        return {}



import aiohttp
import asyncio
import json
import pytest
import okta.models as models

from okta.client import Client as OktaClient


@pytest.mark.asyncio
async def test_set_provisioning_connection(monkeypatch, mocker):
    org_url = "https://test.okta.com"
    token = "TOKEN"
    config = {'orgUrl': org_url, 'token': token}
    client = OktaClient(config)

    # mock http requests, verify if custom header is present in request
    class MockHTTPRequest():
        def __call__(self, **params):
            self.request_info = params
            self.headers = params['headers']
            self.url = params['url']
            self.content_type = 'application/json'
            self.links = ''
            self.text = MockHTTPRequest.mock_response_text
            self.status = 200
            return self

        async def __aenter__(self):
            return self

        async def __aexit__(self, exc_type, exc, tb):
            pass

        @staticmethod
        async def mock_response_text():
            return '[{"text": "mock response text"}]'

    mock_http_request = MockHTTPRequest()
    monkeypatch.setattr(aiohttp.ClientSession, 'request', mock_http_request)

    profile = models.ProvisioningConnectionProfile(
        {
            'authScheme': models.ProvisioningConnectionAuthScheme('TOKEN'),
            'token': 'TEST'
        }
    )
    provisioning_conn_req = models.ProvisioningConnectionRequest({'profile': profile})
    _ = await client.set_default_provisioning_connection_for_application('test_app_id', provisioning_conn_req, query_params={'activate': True})
    assert mock_http_request.request_info['url'].endswith('/apps/test_app_id/connections/default/?activate=True')
    data = mock_http_request.request_info['data']
    assert json.loads(data) == {"profile": {"authScheme": "TOKEN", "token": "TEST"}}


@pytest.mark.asyncio
async def test_list_features_for_application(monkeypatch, mocker):
    mocked_response = """[
        {
            "name": "USER_PROVISIONING",
            "status": "ENABLED",
            "description": "User provisioning settings from Okta to a downstream application",
            "capabilities": {
                "create": {
                    "lifecycleCreate": {
                        "status": "DISABLED"
                    }
                },
                "update": {
                    "profile": {
                        "status": "DISABLED"
                    },
                    "lifecycleDeactivate": {
                        "status": "DISABLED"
                    },
                    "password": {
                        "status": "DISABLED",
                        "seed": "RANDOM",
                        "change": "KEEP_EXISTING"
                    }
                }
            },
            "_links": {
                "self": {
                    "href": "https://${yourOktaDomain}/api/v1/apps/${applicationId}/features/USER_PROVISIONING",
                    "hints": {
                        "allow": [
                            "GET",
                            "PUT"
                        ]
                    }
                }
            }
        }
    ]"""

    org_url = "https://test.okta.com"
    token = "TOKEN"
    config = {'orgUrl': org_url, 'token': token}
    client = OktaClient(config)

    # mock http requests, verify if custom header is present in request
    class MockHTTPRequest():
        def __call__(self, **params):
            self.request_info = params
            self.headers = params['headers']
            self.url = params['url']
            self.content_type = 'application/json'
            self.links = ''
            self.text = MockHTTPRequest.mock_response_text
            self.status = 200
            return self

        async def __aenter__(self):
            return self

        async def __aexit__(self, exc_type, exc, tb):
            pass

        @staticmethod
        async def mock_response_text():
            return mocked_response

    mock_http_request = MockHTTPRequest()
    monkeypatch.setattr(aiohttp.ClientSession, 'request', mock_http_request)

    features, _, err = await client.list_features_for_application('test_app_id')
    assert isinstance(features[0], models.ApplicationFeature)
    assert features[0].name == 'USER_PROVISIONING'
    assert isinstance(features[0].capabilities, models.CapabilitiesObject)
    assert isinstance(features[0].capabilities.create, models.CapabilitiesCreateObject)
    assert isinstance(features[0].capabilities.update, models.CapabilitiesUpdateObject)
    assert isinstance(features[0].capabilities.update, models.CapabilitiesUpdateObject)
    assert isinstance(features[0].capabilities.update.password, models.PasswordSettingObject)
    assert isinstance(features[0].capabilities.update.password.change, models.ChangeEnum)
    assert isinstance(features[0].status, models.EnabledStatus)

@pytest.mark.asyncio
async def test_get_feature_for_application(monkeypatch, mocker):
    mocked_response = """{
        "name": "USER_PROVISIONING",
        "status": "ENABLED",
        "description": "User provisioning settings from Okta to a downstream application",
        "capabilities": {
            "create": {
                "lifecycleCreate": {
                    "status": "DISABLED"
                }
            },
            "update": {
                "profile": {
                    "status": "DISABLED"
                },
                "lifecycleDeactivate": {
                    "status": "DISABLED"
                },
                "password": {
                    "status": "DISABLED",
                    "seed": "RANDOM",
                    "change": "KEEP_EXISTING"
                }
            }
        },
        "_links": {
            "self": {
                "href": "https://${yourOktaDomain}/api/v1/apps/${applicationId}/features/USER_PROVISIONING",
                "hints": {
                    "allow": [
                        "GET",
                        "PUT"
                    ]
                }
            }
        }
    }"""

    org_url = "https://test.okta.com"
    token = "TOKEN"
    config = {'orgUrl': org_url, 'token': token}
    client = OktaClient(config)

    # mock http requests, verify if custom header is present in request
    class MockHTTPRequest():
        def __call__(self, **params):
            self.request_info = params
            self.headers = params['headers']
            self.url = params['url']
            self.content_type = 'application/json'
            self.links = ''
            self.text = MockHTTPRequest.mock_response_text
            self.status = 200
            return self

        async def __aenter__(self):
            return self

        async def __aexit__(self, exc_type, exc, tb):
            pass

        @staticmethod
        async def mock_response_text():
            return mocked_response

    mock_http_request = MockHTTPRequest()
    monkeypatch.setattr(aiohttp.ClientSession, 'request', mock_http_request)

    feature, _, err = await client.get_feature_for_application('test_app_id', 'USER_PROVISIONING')
    assert isinstance(feature, models.ApplicationFeature)
    assert feature.name == 'USER_PROVISIONING'
    assert isinstance(feature.capabilities, models.CapabilitiesObject)
    assert isinstance(feature.capabilities.create, models.CapabilitiesCreateObject)
    assert isinstance(feature.capabilities.update, models.CapabilitiesUpdateObject)
    assert isinstance(feature.capabilities.update, models.CapabilitiesUpdateObject)
    assert isinstance(feature.capabilities.update.password, models.PasswordSettingObject)
    assert isinstance(feature.capabilities.update.password.change, models.ChangeEnum)
    assert isinstance(feature.status, models.EnabledStatus)


@pytest.mark.asyncio
async def test_update_feature_for_application(monkeypatch, mocker):
    org_url = "https://test.okta.com"
    token = "TOKEN"
    config = {'orgUrl': org_url, 'token': token}
    client = OktaClient(config)

    # mock http requests, verify if custom header is present in request
    class MockHTTPRequest():
        def __call__(self, **params):
            self.request_info = params
            self.headers = params['headers']
            self.url = params['url']
            self.content_type = 'application/json'
            self.links = ''
            self.text = MockHTTPRequest.mock_response_text
            self.status = 200
            return self

        async def __aenter__(self):
            return self

        async def __aexit__(self, exc_type, exc, tb):
            pass

        @staticmethod
        async def mock_response_text():
            return '[{"text": "mock response text"}]'

    mock_http_request = MockHTTPRequest()
    monkeypatch.setattr(aiohttp.ClientSession, 'request', mock_http_request)

    capabilities_object_dict = {
        "create": {
            "lifecycleCreate": {
                "status": "ENABLED"
            }
        },
        "update": {
            "lifecycleDeactivate": {
                "status": "ENABLED"
            },
            "profile":{
                "status": "ENABLED"
            },
            "password":{
                "status": "ENABLED",
                "seed": "RANDOM",
                "change": "CHANGE"
            }
        }
    }
    capabilities_object = models.CapabilitiesObject(capabilities_object_dict)

    feature, _, err = await client.update_feature_for_application('test_app_id', 'test_name', capabilities_object)
    assert mock_http_request.request_info['url'].endswith('/apps/test_app_id/features/test_name')
    data = mock_http_request.request_info['data']
    assert json.loads(data) == capabilities_object_dict


@pytest.mark.asyncio
async def test_upload_application_logo(monkeypatch, mocker):
    org_url = "https://test.okta.com"
    token = "TOKEN"
    config = {'orgUrl': org_url, 'token': token}
    client = OktaClient(config)

    # mock http requests, verify if custom header is present in request
    class MockHTTPRequest():
        def __call__(self, **params):
            self.request_info = params
            self.headers = params['headers']
            self.url = params['url']
            self.content_type = 'application/json'
            self.links = ''
            self.text = MockHTTPRequest.mock_response_text
            self.status = 200
            return self

        async def __aenter__(self):
            return self

        async def __aexit__(self, exc_type, exc, tb):
            pass

        @staticmethod
        async def mock_response_text():
            return '[{"text": "mock response text"}]'

    mock_http_request = MockHTTPRequest()
    monkeypatch.setattr(aiohttp.ClientSession, 'request', mock_http_request)

    logo = mocker.Mock()

    _, err = await client.upload_application_logo('test_app_id', logo)

    assert mock_http_request.request_info['url'].endswith('/apps/test_app_id/logo')
    data = mock_http_request.request_info['data']
    assert data == {'file': logo}



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.models.linked_object\
    import LinkedObject
from okta.utils import format_url
from okta.api_client import APIClient


class LinkedObjectClient(APIClient):
    """
    A Client object for the LinkedObject resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_linked_object_definitions(
            self,
            keep_empty_params=False
    ):
        """
        Args:
        Returns:
            list: Collection of LinkedObject instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/user/linkedObjects
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, LinkedObject)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(LinkedObject(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def add_linked_object_definition(
            self, linked_object,
            keep_empty_params=False
    ):
        """
        Args:
            {linked_object}
        Returns:
            LinkedObject
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/user/linkedObjects
            """)

        if isinstance(linked_object, dict):
            body = linked_object
        else:
            body = linked_object.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, LinkedObject)

        if error:
            return (None, response, error)

        try:
            result = LinkedObject(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_linked_object_definition(
            self, linkedObjectName,
            keep_empty_params=False
    ):
        """
        Args:
            linked_object_name {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/user/linkedObjects
                /{linkedObjectName}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_linked_object_definition(
            self, linkedObjectName,
            keep_empty_params=False
    ):
        """
        Args:
            linked_object_name {str}
        Returns:
            LinkedObject
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/user/linkedObjects
                /{linkedObjectName}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, LinkedObject)

        if error:
            return (None, response, error)

        try:
            result = LinkedObject(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



from okta import __version__ as VERSION

import platform


class UserAgent():
    SDK_NAME = "okta-sdk-python"
    PYTHON = "python"

    def __init__(self, user_agent_extra=None):
        python_version = platform.python_version()
        os_name = platform.system()
        os_version = platform.release()
        self._user_agent_string = (f"{UserAgent.SDK_NAME}/{VERSION} "
                                   f"{UserAgent.PYTHON}/{python_version} "
                                   f"{os_name}/{os_version}")
        if (user_agent_extra):
            self._user_agent_string += f" {user_agent_extra}"

    def get_user_agent_string(self):
        return self._user_agent_string



import pytest
from tests.mocks import MockOktaClient
import okta.models as models
from http import HTTPStatus
from okta.errors.okta_api_error import OktaAPIError


class TestTrustedOriginsResource:
    """
    Integration Tests for the Trusted Origins Resource
    """
    SDK_PREFIX = "python_sdk"

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_create_get_origin(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Trusted Origin
        TO_NAME = f"{TestTrustedOriginsResource.SDK_PREFIX}_test_TO"
        TO_ORIGIN = "http://example.com"
        trusted_origin_model = models.TrustedOrigin({
            "name": TO_NAME,
            "origin": TO_ORIGIN,
            "scopes": [
                models.Scope({
                    "type": models.ScopeType.CORS
                }),
                models.Scope({
                    "type": models.ScopeType.REDIRECT
                }),
            ]
        })

        try:
            created_trusted_origin, _, err = await \
                client.create_origin(trusted_origin_model)
            assert err is None
            assert isinstance(created_trusted_origin, models.TrustedOrigin)

            # Retrieve
            retrieved_origin, _, err = await \
                client.get_origin(created_trusted_origin.id)
            assert err is None
            assert isinstance(retrieved_origin, models.TrustedOrigin)
            assert retrieved_origin.name == created_trusted_origin.name
            assert len(retrieved_origin.scopes) == 2

        finally:
            # Delete
            _, err = await client.delete_origin(created_trusted_origin.id)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_origins(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Trusted Origin
        TO_NAME = f"{TestTrustedOriginsResource.SDK_PREFIX}_test_TO"
        TO_ORIGIN = "http://example.com"
        trusted_origin_model = models.TrustedOrigin({
            "name": TO_NAME,
            "origin": TO_ORIGIN,
            "scopes": [
                models.Scope({
                    "type": models.ScopeType.CORS
                }),
                models.Scope({
                    "type": models.ScopeType.REDIRECT
                }),
            ]
        })

        try:
            created_trusted_origin, _, err = await \
                client.create_origin(trusted_origin_model)
            assert err is None
            assert isinstance(created_trusted_origin, models.TrustedOrigin)

            # List
            trusted_origins, _, err = await client.list_origins()
            assert err is None
            assert isinstance(trusted_origins, list)
            assert len(trusted_origins) > 0
            assert isinstance(trusted_origins[0], models.TrustedOrigin)
            assert next((to for to in trusted_origins
                         if to.name == created_trusted_origin.name), None) \
                is not None

        finally:
            # Delete
            _, err = await client.delete_origin(created_trusted_origin.id)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_origin(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Trusted Origin
        TO_NAME = f"{TestTrustedOriginsResource.SDK_PREFIX}_test_TO"
        TO_ORIGIN = "http://example.com"
        trusted_origin_model = models.TrustedOrigin({
            "name": TO_NAME,
            "origin": TO_ORIGIN,
            "scopes": [
                models.Scope({
                    "type": models.ScopeType.CORS
                }),
                models.Scope({
                    "type": models.ScopeType.REDIRECT
                }),
            ]
        })

        try:
            created_trusted_origin, _, err = await \
                client.create_origin(trusted_origin_model)
            assert err is None
            assert isinstance(created_trusted_origin, models.TrustedOrigin)

            # Retrieve
            retrieved_origin, _, err = await \
                client.get_origin(created_trusted_origin.id)
            assert err is None
            assert isinstance(retrieved_origin, models.TrustedOrigin)
            assert retrieved_origin.name == created_trusted_origin.name
            assert len(retrieved_origin.scopes) == 2

            # Delete
            _, err = await client.delete_origin(created_trusted_origin.id)

            # Retrieve to validate
            retrieved_origin, resp, err = await \
                client.get_origin(created_trusted_origin.id)
            assert err is not None
            assert isinstance(err, OktaAPIError)
            assert resp.get_status() == HTTPStatus.NOT_FOUND
            assert retrieved_origin is None
        finally:
            # Clean up
            try:
                _, err = await client.delete_origin(created_trusted_origin.id)
            except Exception:
                pass

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_origin(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Trusted Origin
        TO_NAME = f"{TestTrustedOriginsResource.SDK_PREFIX}_test_TO"
        TO_ORIGIN = "http://example.com"
        trusted_origin_model = models.TrustedOrigin({
            "name": TO_NAME,
            "origin": TO_ORIGIN,
            "scopes": [
                models.Scope({
                    "type": models.ScopeType.CORS
                }),
                models.Scope({
                    "type": models.ScopeType.REDIRECT
                }),
            ]
        })

        try:
            created_trusted_origin, _, err = await \
                client.create_origin(trusted_origin_model)
            assert err is None
            assert isinstance(created_trusted_origin, models.TrustedOrigin)

            # Retrieve
            retrieved_origin, _, err = await \
                client.get_origin(created_trusted_origin.id)
            assert err is None
            assert isinstance(retrieved_origin, models.TrustedOrigin)
            assert retrieved_origin.name == created_trusted_origin.name
            assert len(retrieved_origin.scopes) == 2

            # Update
            updated_trusted_origin_model = models.TrustedOrigin({
                "name": TO_NAME + "_updated",
                "origin": TO_ORIGIN,
                "scopes": [
                    models.Scope({
                        "type": models.ScopeType.CORS
                    }),
                    models.Scope({
                        "type": models.ScopeType.REDIRECT
                    }),
                ]
            })
            updated_origin, _, err = await \
                client.update_origin(created_trusted_origin.id,
                                     updated_trusted_origin_model)
            assert err is None
            assert isinstance(updated_origin, models.TrustedOrigin)
            assert updated_origin.id == created_trusted_origin.id
            assert updated_origin.name == updated_trusted_origin_model.name

            # Retrieve to validate
            retrieved_origin, resp, err = await \
                client.get_origin(created_trusted_origin.id)
            assert retrieved_origin.id == created_trusted_origin.id
            assert retrieved_origin.name == updated_origin.name

        finally:
            # Delete
            _, err = await client.delete_origin(created_trusted_origin.id)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_activate_deactivate_origin(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Trusted Origin
        TO_NAME = f"{TestTrustedOriginsResource.SDK_PREFIX}_test_TO"
        TO_ORIGIN = "http://example.com"
        trusted_origin_model = models.TrustedOrigin({
            "name": TO_NAME,
            "origin": TO_ORIGIN,
            "scopes": [
                models.Scope({
                    "type": models.ScopeType.CORS
                }),
                models.Scope({
                    "type": models.ScopeType.REDIRECT
                }),
            ]
        })

        try:
            created_trusted_origin, _, err = await \
                client.create_origin(trusted_origin_model)
            assert err is None
            assert isinstance(created_trusted_origin, models.TrustedOrigin)
            assert created_trusted_origin.status == "ACTIVE"

            # Deactivate
            deactivated_origin, _, err = await \
                client.deactivate_origin(created_trusted_origin.id)
            assert err is None
            assert deactivated_origin.status == "INACTIVE"

            # Retrieve to validate
            retrieved_origin, resp, err = await \
                client.get_origin(created_trusted_origin.id)
            assert retrieved_origin.id == created_trusted_origin.id
            assert retrieved_origin.status == "INACTIVE"

            # Reactivate
            reactivated_origin, _, err = await \
                client.activate_origin(created_trusted_origin.id)
            assert err is None
            assert reactivated_origin.status == "ACTIVE"

            # Retrieve to validate
            retrieved_origin, resp, err = await \
                client.get_origin(created_trusted_origin.id)
            assert retrieved_origin.id == created_trusted_origin.id
            assert retrieved_origin.status == "ACTIVE"

        finally:
            # Delete
            _, err = await client.delete_origin(created_trusted_origin.id)
            assert err is None



# flake8: noqa
"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.okta_object import OktaObject

from okta.helpers import to_snake_case, to_lower_camel_case


class UserProfile(
    OktaObject
):
    """
    A class for UserProfile objects.
    """

    BASIC_ATTRIBUTES = (
        "city",
        "costCenter",
        "countryCode",
        "department",
        "displayName",
        "division",
        "email",
        "employeeNumber",
        "firstName",
        "honorificPrefix",
        "honorificSuffix",
        "lastName",
        "locale",
        "login",
        "manager",
        "managerId",
        "middleName",
        "mobilePhone",
        "nickName",
        "organization",
        "postalAddress",
        "preferredLanguage",
        "primaryPhone",
        "profileUrl",
        "secondEmail",
        "state",
        "streetAddress",
        "timezone",
        "title",
        "userType",
        "zipCode",
    )

    def __init__(self, config=None):
        super().__init__(config)
        if config:
            self.city = config["city"]\
                if "city" in config else None
            self.cost_center = config["costCenter"]\
                if "costCenter" in config else None
            self.country_code = config["countryCode"]\
                if "countryCode" in config else None
            self.department = config["department"]\
                if "department" in config else None
            self.display_name = config["displayName"]\
                if "displayName" in config else None
            self.division = config["division"]\
                if "division" in config else None
            self.email = config["email"]\
                if "email" in config else None
            self.employee_number = config["employeeNumber"]\
                if "employeeNumber" in config else None
            self.first_name = config["firstName"]\
                if "firstName" in config else None
            self.honorific_prefix = config["honorificPrefix"]\
                if "honorificPrefix" in config else None
            self.honorific_suffix = config["honorificSuffix"]\
                if "honorificSuffix" in config else None
            self.last_name = config["lastName"]\
                if "lastName" in config else None
            self.locale = config["locale"]\
                if "locale" in config else None
            self.login = config["login"]\
                if "login" in config else None
            self.manager = config["manager"]\
                if "manager" in config else None
            self.manager_id = config["managerId"]\
                if "managerId" in config else None
            self.middle_name = config["middleName"]\
                if "middleName" in config else None
            self.mobile_phone = config["mobilePhone"]\
                if "mobilePhone" in config else None
            self.nick_name = config["nickName"]\
                if "nickName" in config else None
            self.organization = config["organization"]\
                if "organization" in config else None
            self.postal_address = config["postalAddress"]\
                if "postalAddress" in config else None
            self.preferred_language = config["preferredLanguage"]\
                if "preferredLanguage" in config else None
            self.primary_phone = config["primaryPhone"]\
                if "primaryPhone" in config else None
            self.profile_url = config["profileUrl"]\
                if "profileUrl" in config else None
            self.second_email = config["secondEmail"]\
                if "secondEmail" in config else None
            self.state = config["state"]\
                if "state" in config else None
            self.street_address = config["streetAddress"]\
                if "streetAddress" in config else None
            self.timezone = config["timezone"]\
                if "timezone" in config else None
            self.title = config["title"]\
                if "title" in config else None
            self.user_type = config["userType"]\
                if "userType" in config else None
            self.zip_code = config["zipCode"]\
                if "zipCode" in config else None
            # set custom attributes not defined in model, do not change string case
            for attr_name in config:
                lower_camel_case = to_lower_camel_case(attr_name)
                if lower_camel_case not in UserProfile.BASIC_ATTRIBUTES:
                    setattr(self, attr_name, config[attr_name])
        else:
            self.city = None
            self.cost_center = None
            self.country_code = None
            self.department = None
            self.display_name = None
            self.division = None
            self.email = None
            self.employee_number = None
            self.first_name = None
            self.honorific_prefix = None
            self.honorific_suffix = None
            self.last_name = None
            self.locale = None
            self.login = None
            self.manager = None
            self.manager_id = None
            self.middle_name = None
            self.mobile_phone = None
            self.nick_name = None
            self.organization = None
            self.postal_address = None
            self.preferred_language = None
            self.primary_phone = None
            self.profile_url = None
            self.second_email = None
            self.state = None
            self.street_address = None
            self.timezone = None
            self.title = None
            self.user_type = None
            self.zip_code = None

    def request_format(self):
        parent_req_format = super().request_format()
        current_obj_format = {
            "city": self.city,
            "costCenter": self.cost_center,
            "countryCode": self.country_code,
            "department": self.department,
            "displayName": self.display_name,
            "division": self.division,
            "email": self.email,
            "employeeNumber": self.employee_number,
            "firstName": self.first_name,
            "honorificPrefix": self.honorific_prefix,
            "honorificSuffix": self.honorific_suffix,
            "lastName": self.last_name,
            "locale": self.locale,
            "login": self.login,
            "manager": self.manager,
            "managerId": self.manager_id,
            "middleName": self.middle_name,
            "mobilePhone": self.mobile_phone,
            "nickName": self.nick_name,
            "organization": self.organization,
            "postalAddress": self.postal_address,
            "preferredLanguage": self.preferred_language,
            "primaryPhone": self.primary_phone,
            "profileUrl": self.profile_url,
            "secondEmail": self.second_email,
            "state": self.state,
            "streetAddress": self.street_address,
            "timezone": self.timezone,
            "title": self.title,
            "userType": self.user_type,
            "zipCode": self.zip_code
        }
        available_attrs = vars(self)
        for attr in available_attrs:
            # do not allow overriding of base attributes
            if not attr in current_obj_format:
                current_obj_format[attr] = getattr(self, attr)
        parent_req_format.update(current_obj_format)
        return parent_req_format

    def __getattr__(self, attr_name):
        """
        Try different cases for backward compatibility.
        """
        available_attrs = vars(self)
        snake_case = to_snake_case(attr_name)
        if snake_case in available_attrs:
            return available_attrs[snake_case]
        lower_camel_case = to_lower_camel_case(attr_name)
        if lower_camel_case in available_attrs:
            return available_attrs[lower_camel_case]
        raise AttributeError(f"'UserProfile' object has no attribute '{attr_name}'")

    def __setattr__(self, attr_name, attr_value):
        """
        Keep custom attributes unchanged and keep backward compatibility for basic attributes.
        """
        lower_camel_case = to_lower_camel_case(attr_name)
        if lower_camel_case in UserProfile.BASIC_ATTRIBUTES:
            super(UserProfile, self).__setattr__(lower_camel_case, attr_value)
        else:
            super(UserProfile, self).__setattr__(attr_name, attr_value)



import copy
import pytest
from tests.mocks import MockOktaClient
import okta.models as models


class TestAuthenticatorsResource:
    """
    Integration Tests for the Authenticators Resource
    """

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_authenticators(self, fs):
        client = MockOktaClient(fs)
        resp, _, err = await client.list_authenticators()
        assert err is None
        assert len(resp) > 0
        assert all([isinstance(elem, models.Authenticator) for elem in resp])
        assert all([isinstance(elem.type, models.AuthenticatorType) for elem in resp])

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_authenticator(self, fs):
        client = MockOktaClient(fs)
        authenticators_list, _, err = await client.list_authenticators()
        assert err is None

        resp, _, err = await client.get_authenticator(authenticators_list[0].id)
        assert err is None
        assert isinstance(resp, models.Authenticator)
        assert resp.as_dict() == authenticators_list[0].as_dict()

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_activate_and_deactivate_authenticator(self, fs):
        client = MockOktaClient(fs)
        authenticators_list, _, err = await client.list_authenticators()
        assert err is None

        app_authenticator = [a for a in authenticators_list if a.type == 'phone'][0]
        try:
            resp, _, err = await client.activate_authenticator(app_authenticator.id)
            assert err is None
            assert isinstance(resp, models.Authenticator)
            assert resp.status == 'ACTIVE'
        finally:
            resp, _, err = await client.deactivate_authenticator(app_authenticator.id)
            assert err is None
            assert isinstance(resp, models.Authenticator)
            assert resp.status == 'INACTIVE'

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_authenticator(self, fs):
        client = MockOktaClient(fs)
        authenticators_list, _, err = await client.list_authenticators()
        assert err is None

        app_authenticator = [a for a in authenticators_list if a.type == 'phone'][0]
        new_authenticator = copy.deepcopy(app_authenticator)
        new_authenticator.settings.allowed_for = models.AllowedForEnum('RECOVERY')
        try:
            updated_authenticator, _, err = await client.update_authenticator(app_authenticator.id, new_authenticator)
            assert err is None
            assert isinstance(updated_authenticator, models.Authenticator)
            assert updated_authenticator.settings.allowed_for == models.AllowedForEnum('RECOVERY')
        finally:
            updated_authenticator, _, err = await client.update_authenticator(app_authenticator.id, app_authenticator)
            assert err is None



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from urllib.parse import urlencode
from okta.models.network_zone\
    import NetworkZone
from okta.utils import format_url
from okta.api_client import APIClient


class NetworkZoneClient(APIClient):
    """
    A Client object for the NetworkZone resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_network_zones(
            self, query_params={},
            keep_empty_params=False
    ):
        """
        Enumerates network zones added to your organization wit
        h pagination. A subset of zones can be returned that ma
        tch a supported filter expression or query.
        Args:
            query_params {dict}: Map of query parameters for request
            [query_params.after] {str}
            [query_params.limit] {str}
            [query_params.filter] {str}
        Returns:
            list: Collection of NetworkZone instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/zones
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, NetworkZone)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(NetworkZone(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_network_zone(
            self, network_zone,
            keep_empty_params=False
    ):
        """
        Adds a new network zone to your Okta organization.
        Args:
            {network_zone}
        Returns:
            NetworkZone
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/zones
            """)

        if isinstance(network_zone, dict):
            body = network_zone
        else:
            body = network_zone.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, NetworkZone)

        if error:
            return (None, response, error)

        try:
            result = NetworkZone(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_network_zone(
            self, zoneId,
            keep_empty_params=False
    ):
        """
        Removes network zone.
        Args:
            zone_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/zones/{zoneId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_network_zone(
            self, zoneId,
            keep_empty_params=False
    ):
        """
        Fetches a network zone from your Okta organization by `
        id`.
        Args:
            zone_id {str}
        Returns:
            NetworkZone
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/zones/{zoneId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, NetworkZone)

        if error:
            return (None, response, error)

        try:
            result = NetworkZone(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_network_zone(
            self, zoneId, network_zone,
            keep_empty_params=False
    ):
        """
        Updates a network zone in your organization.
        Args:
            zone_id {str}
            {network_zone}
        Returns:
            NetworkZone
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/zones/{zoneId}
            """)

        if isinstance(network_zone, dict):
            body = network_zone
        else:
            body = network_zone.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, NetworkZone)

        if error:
            return (None, response, error)

        try:
            result = NetworkZone(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def activate_network_zone(
            self, zoneId,
            keep_empty_params=False
    ):
        """
        Activate Network Zone
        Args:
            zone_id {str}
        Returns:
            NetworkZone
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/zones/{zoneId}/lifecycle/activate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, NetworkZone)

        if error:
            return (None, response, error)

        try:
            result = NetworkZone(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def deactivate_network_zone(
            self, zoneId,
            keep_empty_params=False
    ):
        """
        Deactivates a network zone.
        Args:
            zone_id {str}
        Returns:
            NetworkZone
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/zones/{zoneId}/lifecycle/deactivate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, NetworkZone)

        if error:
            return (None, response, error)

        try:
            result = NetworkZone(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



# flake8: noqa
"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.okta_object import OktaObject
from okta.okta_collection import OktaCollection
from okta.models import acs_endpoint\
    as acs_endpoint
from okta.models import saml_attribute_statement\
    as saml_attribute_statement
from okta.models import sign_on_inline_hook\
    as sign_on_inline_hook
from okta.models import single_logout\
    as single_logout
from okta.models import sp_certificate\
    as sp_certificate


class SamlApplicationSettingsSignOn(
    OktaObject
):
    """
    A class for SamlApplicationSettingsSignOn objects.
    """

    def __init__(self, config=None):
        super().__init__(config)
        if config:
            self.acs_endpoints = OktaCollection.form_list(
                config["acsEndpoints"] if "acsEndpoints"\
                    in config else [],
                acs_endpoint.AcsEndpoint
            )
            self.allow_multiple_acs_endpoints = config["allowMultipleAcsEndpoints"]\
                if "allowMultipleAcsEndpoints" in config else None
            self.assertion_signed = config["assertionSigned"]\
                if "assertionSigned" in config else None
            self.attribute_statements = OktaCollection.form_list(
                config["attributeStatements"] if "attributeStatements"\
                    in config else [],
                saml_attribute_statement.SamlAttributeStatement
            )
            self.audience = config["audience"]\
                if "audience" in config else None
            self.audience_override = config["audienceOverride"]\
                if "audienceOverride" in config else None
            self.authn_context_class_ref = config["authnContextClassRef"]\
                if "authnContextClassRef" in config else None
            self.default_relay_state = config["defaultRelayState"]\
                if "defaultRelayState" in config else None
            self.destination = config["destination"]\
                if "destination" in config else None
            self.destination_override = config["destinationOverride"]\
                if "destinationOverride" in config else None
            self.digest_algorithm = config["digestAlgorithm"]\
                if "digestAlgorithm" in config else None
            self.honor_force_authn = config["honorForceAuthn"]\
                if "honorForceAuthn" in config else None
            self.idp_issuer = config["idpIssuer"]\
                if "idpIssuer" in config else None
            self.inline_hooks = OktaCollection.form_list(
                config["inlineHooks"] if "inlineHooks"\
                    in config else [],
                sign_on_inline_hook.SignOnInlineHook
            )
            self.recipient = config["recipient"]\
                if "recipient" in config else None
            self.recipient_override = config["recipientOverride"]\
                if "recipientOverride" in config else None
            self.request_compressed = config["requestCompressed"]\
                if "requestCompressed" in config else None
            self.response_signed = config["responseSigned"]\
                if "responseSigned" in config else None
            self.signature_algorithm = config["signatureAlgorithm"]\
                if "signatureAlgorithm" in config else None
            if "slo" in config:
                if isinstance(config["slo"],
                              single_logout.SingleLogout):
                    self.slo = config["slo"]
                elif config["slo"] is not None:
                    self.slo = single_logout.SingleLogout(
                        config["slo"]
                    )
                else:
                    self.slo = None
            else:
                self.slo = None
            if "spCertificate" in config:
                if isinstance(config["spCertificate"],
                              sp_certificate.SpCertificate):
                    self.sp_certificate = config["spCertificate"]
                elif config["spCertificate"] is not None:
                    self.sp_certificate = sp_certificate.SpCertificate(
                        config["spCertificate"]
                    )
                else:
                    self.sp_certificate = None
            else:
                self.sp_certificate = None
            self.sp_issuer = config["spIssuer"]\
                if "spIssuer" in config else None
            self.sso_acs_url = config["ssoAcsUrl"]\
                if "ssoAcsUrl" in config else None
            self.sso_acs_url_override = config["ssoAcsUrlOverride"]\
                if "ssoAcsUrlOverride" in config else None
            self.subject_name_id_format = config["subjectNameIdFormat"]\
                if "subjectNameIdFormat" in config else None
            self.subject_name_id_template = config["subjectNameIdTemplate"]\
                if "subjectNameIdTemplate" in config else None
        else:
            self.acs_endpoints = []
            self.allow_multiple_acs_endpoints = None
            self.assertion_signed = None
            self.attribute_statements = []
            self.audience = None
            self.audience_override = None
            self.authn_context_class_ref = None
            self.default_relay_state = None
            self.destination = None
            self.destination_override = None
            self.digest_algorithm = None
            self.honor_force_authn = None
            self.idp_issuer = None
            self.inline_hooks = []
            self.recipient = None
            self.recipient_override = None
            self.request_compressed = None
            self.response_signed = None
            self.signature_algorithm = None
            self.slo = None
            self.sp_certificate = None
            self.sp_issuer = None
            self.sso_acs_url = None
            self.sso_acs_url_override = None
            self.subject_name_id_format = None
            self.subject_name_id_template = None

    def request_format(self):
        parent_req_format = super().request_format()
        current_obj_format = {
            "acsEndpoints": self.acs_endpoints,
            "allowMultipleAcsEndpoints": self.allow_multiple_acs_endpoints,
            "assertionSigned": self.assertion_signed,
            "attributeStatements": self.attribute_statements,
            "audience": self.audience,
            "audienceOverride": self.audience_override,
            "authnContextClassRef": self.authn_context_class_ref,
            "defaultRelayState": self.default_relay_state,
            "destination": self.destination,
            "destinationOverride": self.destination_override,
            "digestAlgorithm": self.digest_algorithm,
            "honorForceAuthn": self.honor_force_authn,
            "idpIssuer": self.idp_issuer,
            "inlineHooks": self.inline_hooks,
            "recipient": self.recipient,
            "recipientOverride": self.recipient_override,
            "requestCompressed": self.request_compressed,
            "responseSigned": self.response_signed,
            "signatureAlgorithm": self.signature_algorithm,
            "slo": self.slo,
            "spCertificate": self.sp_certificate,
            "spIssuer": self.sp_issuer,
            "ssoAcsUrl": self.sso_acs_url,
            "ssoAcsUrlOverride": self.sso_acs_url_override,
            "subjectNameIdFormat": self.subject_name_id_format,
            "subjectNameIdTemplate": self.subject_name_id_template
        }
        parent_req_format.update(current_obj_format)
        return parent_req_format



import pytest
from tests.mocks import MockOktaClient
import okta.models as models
from http import HTTPStatus
from okta.errors.okta_api_error import OktaAPIError


class TestAuthorizationServerResource:
    """
    Integration Tests for the Authorization Server Resource
    """
    SDK_PREFIX = "python_sdk"

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_create_auth_server(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_aaPeZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_auth_servers(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_aaPeZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # List Auth Servers
            auth_servers, _, err = await client.list_authorization_servers()
            assert err is None
            # Assuming default Auth server instantiated
            assert auth_servers is not None
            assert len(auth_servers) > 0
            assert next(
                (server for server in auth_servers
                 if server.id == created_auth_server.id))

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_created_auth_server(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_aaPeZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Get Auth Server
            retrieved_auth_server, _, err = await\
                client.get_authorization_server(created_auth_server.id)
            assert err is None
            assert retrieved_auth_server.id ==\
                created_auth_server.id
            assert retrieved_auth_server.name ==\
                created_auth_server.name
            assert retrieved_auth_server.description ==\
                created_auth_server.description

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_auth_server(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abPeZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] ==\
                TEST_AUDS[0]

            # Update
            UPDATED_DESC = f"{TEST_DESC}-Updated"
            created_auth_server.description = UPDATED_DESC
            updated_auth_server, _, err = await client.update_authorization_server(
                created_auth_server.id,
                created_auth_server)
            assert err is None
            assert updated_auth_server.description == UPDATED_DESC
            assert updated_auth_server.id == created_auth_server.id

            # Get Auth Server
            retrieved_server, _, err = await\
                client.get_authorization_server(created_auth_server.id)
            assert err is None
            assert retrieved_server.id == updated_auth_server.id
            assert retrieved_server.name == updated_auth_server.name
            assert retrieved_server.description == updated_auth_server.description

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_deactivate_delete_server(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abCeZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Get Auth Server
            retrieved_auth_server, _, err = await\
                client.get_authorization_server(created_auth_server.id)
            assert err is None
            assert retrieved_auth_server.id == created_auth_server.id
            assert retrieved_auth_server.status == "ACTIVE"

            # DeActivate Auth server
            _, err = await client.deactivate_authorization_server(
                created_auth_server.id)
            assert err is None

            # Get Auth Server
            retrieved_auth_server, _, err = await\
                client.get_authorization_server(created_auth_server.id)
            assert err is None
            assert retrieved_auth_server.id == created_auth_server.id
            assert retrieved_auth_server.status == "INACTIVE"

            # Delete Auth server
            _, err = await client.delete_authorization_server(
                created_auth_server.id)
            assert err is None

            # Get Auth Server
            retrieved_auth_server, resp, err = await\
                client.get_authorization_server(created_auth_server.id)
            assert err is not None
            assert isinstance(err, OktaAPIError)
            assert resp.get_status() == HTTPStatus.NOT_FOUND

        finally:
            # Clean up if auth server wasn't deleted during test
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
            except Exception:
                pass
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
            except Exception:
                pass

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_activate_server(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abCeZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Get Auth Server
            retrieved_auth_server, _, err = await\
                client.get_authorization_server(created_auth_server.id)
            assert err is None
            assert retrieved_auth_server.id == created_auth_server.id
            assert retrieved_auth_server.status == "ACTIVE"

            # DeActivate Auth server
            _, err = await client.deactivate_authorization_server(
                created_auth_server.id)
            assert err is None

            # Get Auth Server
            retrieved_auth_server, _, err = await\
                client.get_authorization_server(created_auth_server.id)
            assert err is None
            assert retrieved_auth_server.id == created_auth_server.id
            assert retrieved_auth_server.status == "INACTIVE"

            # Activate Auth server
            _, err = await client.activate_authorization_server(
                created_auth_server.id)
            assert err is None

            # Get Auth Server
            retrieved_auth_server, _, err = await\
                client.get_authorization_server(created_auth_server.id)
            assert err is None
            assert retrieved_auth_server.id == created_auth_server.id
            assert retrieved_auth_server.status == "ACTIVE"

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_auth_server_policies(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDeZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Policy
            POLICY_TYPE = models.PolicyType.OAUTH_AUTHORIZATION_POLICY
            POLICY_STATUS = "ACTIVE"
            POLICY_NAME = "Test Policy"
            POLICY_DESC = "Test Policy"
            POLICY_PRIORITY = 1
            POLICY_CONDITIONS = models.PolicyRuleConditions({
                "clients": models.ClientPolicyCondition({
                    "include": ["ALL_CLIENTS"]
                })
            })

            policy_model = models.Policy({
                "type": POLICY_TYPE,
                "status": POLICY_STATUS,
                "name": POLICY_NAME,
                "description": POLICY_DESC,
                "priority": POLICY_PRIORITY,
                "conditions": POLICY_CONDITIONS
            })

            created_policy, _, err = await\
                client.create_authorization_server_policy(
                    created_auth_server.id, policy_model
                )
            assert err is None

            # Get Policies
            policies, _, err = await client.list_authorization_server_policies(
                created_auth_server.id
            )
            assert err is None
            assert len(policies) > 0
            assert next((plcy for plcy in policies
                         if plcy.id == created_policy.id))

            # Delete Policy
            _, err = await client.delete_authorization_server_policy(
                created_auth_server.id, created_policy.id
            )
            assert err is None

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_auth_server_policy(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Policy
            POLICY_TYPE = models.PolicyType.OAUTH_AUTHORIZATION_POLICY
            POLICY_STATUS = "ACTIVE"
            POLICY_NAME = "Test Policy"
            POLICY_DESC = "Test Policy"
            POLICY_PRIORITY = 1
            POLICY_CONDITIONS = models.PolicyRuleConditions({
                "clients": models.ClientPolicyCondition({
                    "include": ["ALL_CLIENTS"]
                })
            })

            policy_model = models.Policy({
                "type": POLICY_TYPE,
                "status": POLICY_STATUS,
                "name": POLICY_NAME,
                "description": POLICY_DESC,
                "priority": POLICY_PRIORITY,
                "conditions": POLICY_CONDITIONS
            })

            created_policy, _, err = await\
                client.create_authorization_server_policy(
                    created_auth_server.id, policy_model
                )
            assert err is None

            # Get Policy
            found_policy, _, err = await client.get_authorization_server_policy(
                created_auth_server.id, created_policy.id
            )
            assert err is None
            assert found_policy.id == created_policy.id
            assert found_policy.name == created_policy.name
            assert found_policy.description == created_policy.description

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_authorization_server_policy(
                    created_auth_server.id, created_policy.id
                )
                assert err is None
            except Exception as exc:
                errors.append(exc)

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_auth_server_policy(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Policy
            POLICY_TYPE = models.PolicyType.OAUTH_AUTHORIZATION_POLICY
            POLICY_STATUS = "ACTIVE"
            POLICY_NAME = "Test Policy"
            POLICY_DESC = "Test Policy"
            POLICY_PRIORITY = 1
            POLICY_CONDITIONS = models.PolicyRuleConditions({
                "clients": models.ClientPolicyCondition({
                    "include": ["ALL_CLIENTS"]
                })
            })

            policy_model = models.Policy({
                "type": POLICY_TYPE,
                "status": POLICY_STATUS,
                "name": POLICY_NAME,
                "description": POLICY_DESC,
                "priority": POLICY_PRIORITY,
                "conditions": POLICY_CONDITIONS
            })

            created_policy, _, err = await\
                client.create_authorization_server_policy(
                    created_auth_server.id, policy_model
                )
            assert err is None

            # Update policy
            UPDATED_NAME = f"{POLICY_NAME}_updated"
            UPDATED_DESC = f"{POLICY_DESC}_updated"
            created_policy.name = UPDATED_NAME
            created_policy.description = UPDATED_DESC

            updated_auth_server_policy, _, err = await\
                client.update_authorization_server_policy(
                    created_auth_server.id, created_policy.id, created_policy
                )
            assert err is None
            assert updated_auth_server_policy.name == UPDATED_NAME
            assert updated_auth_server_policy.description == UPDATED_DESC

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_authorization_server_policy(
                    created_auth_server.id, created_policy.id
                )
                assert err is None
            except Exception as exc:
                errors.append(exc)

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_auth_server_policy(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Policy
            POLICY_TYPE = models.PolicyType.OAUTH_AUTHORIZATION_POLICY
            POLICY_STATUS = "ACTIVE"
            POLICY_NAME = "Test Policy"
            POLICY_DESC = "Test Policy"
            POLICY_PRIORITY = 1
            POLICY_CONDITIONS = models.PolicyRuleConditions({
                "clients": models.ClientPolicyCondition({
                    "include": ["ALL_CLIENTS"]
                })
            })

            policy_model = models.Policy({
                "type": POLICY_TYPE,
                "status": POLICY_STATUS,
                "name": POLICY_NAME,
                "description": POLICY_DESC,
                "priority": POLICY_PRIORITY,
                "conditions": POLICY_CONDITIONS
            })

            created_policy, _, err = await\
                client.create_authorization_server_policy(
                    created_auth_server.id, policy_model
                )
            assert err is None

            # Get Policy
            found_policy, _, err = await\
                client.get_authorization_server_policy(
                    created_auth_server.id, created_policy.id
                )
            assert err is None
            assert found_policy is not None
            assert found_policy.id == created_policy.id
            assert found_policy.name == created_policy.name
            assert found_policy.description == created_policy.description

            # Delete Policy
            _, err = await client.delete_authorization_server_policy(
                created_auth_server.id, created_policy.id
            )
            assert err is None

            # Get Policy
            found_policy, resp, err = await\
                client.get_authorization_server_policy(
                    created_auth_server.id, created_policy.id
                )
            assert err is not None
            assert isinstance(err, OktaAPIError)
            assert resp.get_status() == HTTPStatus.NOT_FOUND
            assert found_policy is None

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_authorization_server_policy(
                    created_auth_server.id, created_policy.id
                )
            except Exception:
                pass

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_create_get_oauth2_scope(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Oauth Scope
            SCOPE_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}:abDGz"
            scope_obj = models.OAuth2Scope({
                "name": SCOPE_NAME
            })

            oauth_scope, _, err = await client.create_o_auth_2_scope(
                created_auth_server.id, scope_obj
            )
            assert err is None
            assert isinstance(oauth_scope, models.OAuth2Scope)
            assert oauth_scope.name == scope_obj.name

            # Get Oauth Scope
            found_oauth_scope, _, err = await client.get_o_auth_2_scope(
                created_auth_server.id, oauth_scope.id
            )
            assert err is None
            assert found_oauth_scope.name == oauth_scope.name

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_o_auth_2_scope(
                    created_auth_server.id, oauth_scope.id
                )
                assert err is None
            except Exception as exc:
                errors.append(exc)

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_oauth2_scopes(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDgZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Oauth Scope
            SCOPE_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}:abDGz"
            scope_obj = models.OAuth2Scope({
                "name": SCOPE_NAME
            })

            oauth_scope, _, err = await client.create_o_auth_2_scope(
                created_auth_server.id, scope_obj
            )
            assert err is None
            assert isinstance(oauth_scope, models.OAuth2Scope)
            assert oauth_scope.name == scope_obj.name

            # List Oauth Scopes
            scopes, _, err = await client.list_o_auth_2_scopes(
                created_auth_server.id)
            assert err is None
            assert len(scopes) > 0
            assert next((scope for scope in scopes if scope.id == oauth_scope.id))

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_o_auth_2_scope(
                    created_auth_server.id, oauth_scope.id
                )
                assert err is None
            except Exception as exc:
                errors.append(exc)

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_oauth2_scope(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Oauth Scope
            SCOPE_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}:abDHz"
            scope_obj = models.OAuth2Scope({
                "name": SCOPE_NAME,
                "consent": "REQUIRED",
                "metadataPublish": "ALL_CLIENTS"
            })
            UPDATED_SCOPE_NAME = f"{SCOPE_NAME}updated"
            updated_obj = models.OAuth2Scope({
                "name": UPDATED_SCOPE_NAME,
                "consent": "REQUIRED",
                "metadataPublish": "ALL_CLIENTS"
            })

            oauth_scope, _, err = await client.create_o_auth_2_scope(
                created_auth_server.id, scope_obj
            )
            assert err is None
            assert isinstance(oauth_scope, models.OAuth2Scope)
            assert oauth_scope.name == scope_obj.name

            # Update scope
            updated_scope, _, err = await client.update_o_auth_2_scope(
                created_auth_server.id, oauth_scope.id, updated_obj
            )
            assert err is None
            assert updated_scope.name == UPDATED_SCOPE_NAME

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_o_auth_2_scope(
                    created_auth_server.id, oauth_scope.id
                )
                assert err is None
            except Exception as exc:
                errors.append(exc)

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_oauth2_scope(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Oauth Scope
            SCOPE_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}:abDGz"
            scope_obj = models.OAuth2Scope({
                "name": SCOPE_NAME
            })

            oauth_scope, _, err = await client.create_o_auth_2_scope(
                created_auth_server.id, scope_obj
            )
            assert err is None
            assert isinstance(oauth_scope, models.OAuth2Scope)
            assert oauth_scope.name == scope_obj.name

            # Get Oauth Scope
            found_oauth_scope, _, err = await client.get_o_auth_2_scope(
                created_auth_server.id, oauth_scope.id
            )
            assert err is None
            assert found_oauth_scope.name == oauth_scope.name

            # Delete Oauth Scope
            _, err = await client.delete_o_auth_2_scope(
                created_auth_server.id, oauth_scope.id
            )
            assert err is None

            # Get Oauth Scope
            found_oauth_scope, resp, err = await client.get_o_auth_2_scope(
                created_auth_server.id, oauth_scope.id
            )
            assert err is not None
            assert isinstance(err, OktaAPIError)
            assert resp.get_status() == HTTPStatus.NOT_FOUND

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_o_auth_2_scope(
                    created_auth_server.id, oauth_scope.id
                )
            except Exception:
                pass

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_create_get_oauth2_claim(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Oauth Claim
            CLAIM_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_abeIz"
            CLAIM_STATUS = "INACTIVE"
            CLAIM_TYPE = "RESOURCE"
            CLAIM_VALUE_TYPE = "EXPRESSION"
            CLAIM_VALUE = "\"driving!\""
            claim_obj = models.OAuth2Claim({
                "name": CLAIM_NAME,
                "status": CLAIM_STATUS,
                "claimType": CLAIM_TYPE,
                "valueType": CLAIM_VALUE_TYPE,
                "value": CLAIM_VALUE
            })

            oauth_claim, _, err = await client.create_o_auth_2_claim(
                created_auth_server.id, claim_obj
            )
            assert err is None
            assert isinstance(oauth_claim, models.OAuth2Claim)
            assert oauth_claim.name == CLAIM_NAME

            # Get Oauth claim
            found_oauth_claim, _, err = await client.get_o_auth_2_claim(
                created_auth_server.id, oauth_claim.id
            )
            assert err is None
            assert found_oauth_claim.name == oauth_claim.name

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_o_auth_2_claim(
                    created_auth_server.id, oauth_claim.id
                )
                assert err is None
            except Exception as exc:
                errors.append(exc)

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_oauth2_claims(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Oauth Claim
            CLAIM_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_abeIz"
            CLAIM_STATUS = "INACTIVE"
            CLAIM_TYPE = "RESOURCE"
            CLAIM_VALUE_TYPE = "EXPRESSION"
            CLAIM_VALUE = "\"driving!\""
            claim_obj = models.OAuth2Claim({
                "name": CLAIM_NAME,
                "status": CLAIM_STATUS,
                "claimType": CLAIM_TYPE,
                "valueType": CLAIM_VALUE_TYPE,
                "value": CLAIM_VALUE
            })

            oauth_claim, _, err = await client.create_o_auth_2_claim(
                created_auth_server.id, claim_obj
            )
            assert err is None
            assert isinstance(oauth_claim, models.OAuth2Claim)
            assert oauth_claim.name == CLAIM_NAME

            # List Oauth claims
            claims, _, err = await client.list_o_auth_2_claims(
                created_auth_server.id)
            assert err is None
            assert next((claim for claim in claims
                         if claim.id == oauth_claim.id))

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_o_auth_2_claim(
                    created_auth_server.id, oauth_claim.id
                )
                assert err is None
            except Exception as exc:
                errors.append(exc)

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_oauth2_claim(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Oauth Claim
            CLAIM_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_abeIz"
            CLAIM_STATUS = "INACTIVE"
            CLAIM_TYPE = "RESOURCE"
            CLAIM_VALUE_TYPE = "EXPRESSION"
            CLAIM_VALUE = "\"driving!\""
            claim_obj = models.OAuth2Claim({
                "name": CLAIM_NAME,
                "status": CLAIM_STATUS,
                "claimType": CLAIM_TYPE,
                "valueType": CLAIM_VALUE_TYPE,
                "value": CLAIM_VALUE
            })
            UPDATED_CLAIM_NAME = f"{CLAIM_NAME}_updated"
            updated_obj = models.OAuth2Claim({
                "name": UPDATED_CLAIM_NAME,
                "status": CLAIM_STATUS,
                "claimType": CLAIM_TYPE,
                "valueType": CLAIM_VALUE_TYPE,
                "value": CLAIM_VALUE
            })

            oauth_claim, _, err = await client.create_o_auth_2_claim(
                created_auth_server.id, claim_obj
            )
            assert err is None
            assert isinstance(oauth_claim, models.OAuth2Claim)
            assert oauth_claim.name == CLAIM_NAME

            # Update claim
            updated_claim, _, err = await client.update_o_auth_2_claim(
                created_auth_server.id, oauth_claim.id, updated_obj
            )
            assert err is None
            assert isinstance(oauth_claim, models.OAuth2Claim)
            assert updated_claim.name == UPDATED_CLAIM_NAME

            # Get Oauth claim
            found_oauth_claim, _, err = await client.get_o_auth_2_claim(
                created_auth_server.id, oauth_claim.id
            )
            assert err is None
            assert found_oauth_claim.name == UPDATED_CLAIM_NAME

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_o_auth_2_claim(
                    created_auth_server.id, oauth_claim.id
                )
                assert err is None
            except Exception as exc:
                errors.append(exc)

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_oauth2_claim(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Oauth Claim
            CLAIM_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_abeIz"
            CLAIM_STATUS = "INACTIVE"
            CLAIM_TYPE = "RESOURCE"
            CLAIM_VALUE_TYPE = "EXPRESSION"
            CLAIM_VALUE = "\"driving!\""
            claim_obj = models.OAuth2Claim({
                "name": CLAIM_NAME,
                "status": CLAIM_STATUS,
                "claimType": CLAIM_TYPE,
                "valueType": CLAIM_VALUE_TYPE,
                "value": CLAIM_VALUE
            })

            oauth_claim, _, err = await client.create_o_auth_2_claim(
                created_auth_server.id, claim_obj
            )
            assert err is None
            assert isinstance(oauth_claim, models.OAuth2Claim)
            assert oauth_claim.name == CLAIM_NAME

            # Get Oauth claim
            found_oauth_claim, _, err = await client.get_o_auth_2_claim(
                created_auth_server.id, oauth_claim.id
            )
            assert err is None
            assert found_oauth_claim.name == oauth_claim.name

            # Delete Oauth claim
            _, err = await client.delete_o_auth_2_claim(
                created_auth_server.id, oauth_claim.id
            )
            assert err is None

            # Get Oauth claim
            found_oauth_claim, resp, err = await client.get_o_auth_2_claim(
                created_auth_server.id, oauth_claim.id
            )
            assert err is not None
            assert isinstance(err, OktaAPIError)
            assert resp.get_status() == HTTPStatus.NOT_FOUND

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.delete_o_auth_2_claim(
                    created_auth_server.id, oauth_claim.id
                )
            except Exception:
                pass

            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_auth_server_keys(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # List keys
            auth_server_keys, _, err = await\
                client.list_authorization_server_keys(
                    created_auth_server.id
                )
            assert err is None
            assert len(auth_server_keys) > 0

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_rotate_auth_server_keys(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDfZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Make key to rotate
            rotate_key = models.JwkUse({
                "use": "sig"
            })

            # Rotate key
            keys, _, err = await client.rotate_authorization_server_keys(
                created_auth_server.id, rotate_key
            )
            assert err is None
            assert len(keys) > 0

        finally:
            errors = []
            # Clean up
            try:
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_authorization_server_policy_rules(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create Auth Server
        TEST_NAME = f"{TestAuthorizationServerResource.SDK_PREFIX}_test_abDeZ"
        TEST_DESC = "Test Auth Server"
        TEST_AUDS = ["api://default"]
        auth_server_model = models.AuthorizationServer({
            "name": TEST_NAME,
            "description": TEST_DESC,
            "audiences": TEST_AUDS
        })

        try:
            created_auth_server, _, err = await \
                client.create_authorization_server(auth_server_model)
            assert err is None
            assert isinstance(created_auth_server, models.AuthorizationServer)
            assert created_auth_server.name == TEST_NAME
            assert created_auth_server.description == TEST_DESC
            assert created_auth_server.audiences == TEST_AUDS
            assert created_auth_server.audiences[0] == TEST_AUDS[0]

            # Create Policy
            POLICY_TYPE = models.PolicyType.OAUTH_AUTHORIZATION_POLICY
            POLICY_STATUS = "ACTIVE"
            POLICY_NAME = "Test Policy"
            POLICY_DESC = "Test Policy"
            POLICY_PRIORITY = 1
            POLICY_CONDITIONS = models.PolicyRuleConditions({
                "clients": models.ClientPolicyCondition({
                    "include": ["ALL_CLIENTS"]
                })
            })

            policy_model = models.Policy({
                "type": POLICY_TYPE,
                "status": POLICY_STATUS,
                "name": POLICY_NAME,
                "description": POLICY_DESC,
                "priority": POLICY_PRIORITY,
                "conditions": POLICY_CONDITIONS
            })

            created_policy, _, err = await\
                client.create_authorization_server_policy(
                    created_auth_server.id, policy_model
                )
            assert err is None

            POLICY_RULE_ACTIONS = models.AuthorizationServerPolicyRuleActions({
                "token": {
                    "accessTokenLifetimeMinutes": 60,
                    "refreshTokenLifetimeMinutes": 0,
                    "refreshTokenWindowMinutes": 10080
                }
            })
            POLICY_RULE_CONDITIONS = models.AuthorizationServerPolicyRuleConditions({
                "people": models.PolicyPeopleCondition({
                    "include": ["EVERYONE"]
                }),
                "grantTypes": models.GrantTypePolicyRuleCondition({
                    "include": [
                        "client_credentials",
                    ]
                }),
                "scopes":{"include": ["*"]}
            })
            policy_rule_model = models.AuthorizationServerPolicyRule({
                "type": "RESOURCE_ACCESS",
                "name": "Test Policy Rule",
                "priority": 1,
                "actions": POLICY_RULE_ACTIONS,
                "conditions": POLICY_RULE_CONDITIONS,
            })

            created_policy_rule, _, err = await \
                client.create_authorization_server_policy_rule(
                    created_policy.id, created_auth_server.id, policy_rule_model
                )
            assert err is None

            # Get Policy Rules
            policy_rules, _, err = await client.list_authorization_server_policy_rules(
                created_policy.id, created_auth_server.id
            )

            assert err is None
            assert len(policy_rules) > 0
            assert next((plcy_rule for plcy_rule in policy_rules
                         if plcy_rule.id == created_policy_rule.id))

        finally:
            # Clear everything despite of any error
            errors = []
            try:
                # Delete Policy Rule
                _, err = await client.delete_authorization_server_policy_rule(
                    created_policy.id, created_auth_server.id, created_policy_rule.id
                )
                if err:
                    errors.append(err)
            except Exception as exc:
                errors.append(exc)

            try:
                # Delete Policy
                _, err = await client.delete_authorization_server_policy(
                    created_auth_server.id, created_policy.id
                )
                if err:
                    errors.append(err)
            except Exception as exc:
                errors.append(exc)

            try:
                # DeActivate Auth server
                _, err = await client.deactivate_authorization_server(
                    created_auth_server.id)
                if err:
                    errors.append(err)
            except Exception as exc:
                errors.append(exc)

            try:
                # Delete Auth server
                _, err = await client.delete_authorization_server(
                    created_auth_server.id)
                if err:
                    errors.append(err)
            except Exception as exc:
                errors.append(exc)

            assert len(errors) == 0



import pytest
from tests.mocks import MockOktaClient
import okta.models as models


class TestFeaturesResource:
    """
    Integration Tests for the Features Resource
    """
    SDK_PREFIX = "python_sdk"

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_features(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # List
        features, _, err = await client.list_features()
        assert err is None
        assert len(features) > 0
        assert isinstance(features[0], models.Feature)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_feature(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # List
        features, _, err = await client.list_features()
        assert err is None
        assert len(features) > 0
        assert isinstance(features[0], models.Feature)

        first_feature = features[0]

        # Retrieve with ID
        retrieved_feature, _, err = await client.get_feature(first_feature.id)
        assert err is None
        assert retrieved_feature.id == first_feature.id
        assert retrieved_feature.name == first_feature.name
        assert retrieved_feature.description == first_feature.description

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_feature_dependencies(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # List
        features, _, err = await client.list_features()
        assert err is None
        assert len(features) > 0
        assert isinstance(features[0], models.Feature)

        first_feature = features[0]

        # Retrieve Dependencies with ID
        feature_dependencies, _, err = await \
            client.list_feature_dependencies(first_feature.id)
        assert err is None
        assert isinstance(feature_dependencies, list)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_feature_dependents(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # List
        features, _, err = await client.list_features()
        assert err is None
        assert len(features) > 0
        assert isinstance(features[0], models.Feature)

        first_feature = features[0]

        # Retrieve Dependencies with ID
        feature_dependents, _, err = await \
            client.list_feature_dependents(first_feature.id)
        assert err is None
        assert isinstance(feature_dependents, list)

    @pytest.mark.skip
    @pytest.mark.asyncio
    async def test_update_feature_lifecycle(self):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # List
        features, _, err = await client.list_features()
        assert err is None
        assert len(features) > 0
        assert isinstance(features[0], models.Feature)

        first_feature = features[0]

        # Update
        first_feature_id = first_feature.id
        original_status = first_feature.status
        lifecycle = "enable" if original_status == "DISABLED"\
            else "disable"
        status_change, _, err = await \
            client.update_feature_lifecycle(first_feature_id, lifecycle)
        assert err is None
        assert isinstance(status_change, models.Feature)
        assert original_status != status_change.status

        # Revert
        lifecycle = "disable" if original_status == "DISABLED"\
            else "enable"
        updated_status_change, _, err = await \
            client.update_feature_lifecycle(first_feature_id, lifecycle)
        assert err is None
        assert isinstance(updated_status_change, models.Feature)
        assert original_status == updated_status_change.status



import asyncio
import copy

from okta.client import Client
from okta.models import Application, SamlApplication, ApplicationSignOnMode
from okta.request_executor import RequestExecutor


SAML_APP_RESP_DICT = {
    "id": "test_id",
    "name": "test_org_custom_saml_app",
    "label": "Custom Saml 2.0 App",
    "status": "ACTIVE",
    "lastUpdated": "2021-02-10T14:21:41.000Z",
    "created": "2021-02-10T14:21:41.000Z",
    "accessibility": {
        "selfService": False,
        "errorRedirectUrl": None,
        "loginRedirectUrl": None,
    },
    "visibility": {
        "autoSubmitToolbar": False,
        "hide": {"iOS": False, "web": False},
        "appLinks": {"test_org_custom_saml_app_link": True},
    },
    "features": [],
    "signOnMode": "SAML_2_0",
    "credentials": {
        "userNameTemplate": {
            "template": '${fn:substringBefore(source.login, "@")}',
            "type": "BUILT_IN",
        },
        "signing": {"kid": "test_kid"},
    },
    "settings": {
        "app": {},
        "notifications": {
            "vpn": {
                "network": {"connection": "DISABLED"},
                "message": None,
                "helpUrl": None,
            }
        },
        "signOn": {
            "defaultRelayState": "",
            "ssoAcsUrl": "http://example.okta.com",
            "idpIssuer": "http://www.okta.com/${org.externalKey}",
            "audience": "https://example.com/tenant/123",
            "recipient": "http://recipient.okta.com",
            "destination": "http://destination.okta.com",
            "subjectNameIdTemplate": "${user.userName}",
            "subjectNameIdFormat": "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress",
            "responseSigned": True,
            "assertionSigned": True,
            "signatureAlgorithm": "RSA_SHA256",
            "digestAlgorithm": "SHA256",
            "honorForceAuthn": True,
            "authnContextClassRef": "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport",
            "spIssuer": None,
            "requestCompressed": False,
            "attributeStatements": [],
            "allowMultipleAcsEndpoints": False,
            "acsEndpoints": [],
            "slo": {"enabled": False},
        },
    },
    "_links": {
        "help": {"href": "https://test_help_link", "type": "text/html"},
        "metadata": {
            "href": "https://test_metadata_link",
            "type": "application/xml",
        },
        "appLinks": [
            {
                "name": "test_app_link",
                "href": "https://test_app_link",
                "type": "text/html",
            }
        ],
        "groups": {"href": "https://test_groups_link"},
        "logo": [
            {
                "name": "medium",
                "href": "https://test_logo_link",
                "type": "image/png",
            }
        ],
        "users": {"href": "https://test_users_link"},
        "deactivate": {"href": "https://test_deactivate_link"},
    },
}


EXPECTED_SAML_APP_AS_DICT = {
    "_links": {
        "help": {"href": "https://test_help_link", "type": "text/html"},
        "metadata": {
            "href": "https://test_metadata_link",
            "type": "application/xml",
        },
        "appLinks": [
            {
                "name": "test_app_link",
                "href": "https://test_app_link",
                "type": "text/html",
            }
        ],
        "groups": {"href": "https://test_groups_link"},
        "logo": [
            {
                "name": "medium",
                "href": "https://test_logo_link",
                "type": "image/png",
            }
        ],
        "users": {"href": "https://test_users_link"},
        "deactivate": {"href": "https://test_deactivate_link"},
    },
    "accessibility": {"selfService": False},
    "created": "2021-02-10T14:21:41.000Z",
    "credentials": {
        "signing": {"kid": "test_kid"},
        "userNameTemplate": {
            "template": '${fn:substringBefore(source.login, "@")}',
            "type": "BUILT_IN",
        },
    },
    "features": [],
    "id": "test_id",
    "label": "Custom Saml 2.0 App",
    "lastUpdated": "2021-02-10T14:21:41.000Z",
    "name": "test_org_custom_saml_app",
    "settings": {
        "app": {},
        "notifications": {
            "vpn": {"network": {"connection": "DISABLED", "exclude": [], "include": []}}
        },
        "signOn": {
            "acsEndpoints": [],
            "allowMultipleAcsEndpoints": False,
            "assertionSigned": True,
            "attributeStatements": [],
            "audience": "https://example.com/tenant/123",
            "authnContextClassRef": "urn:oasis:names:tc:SAML:2.0:ac:classes:PasswordProtectedTransport",
            "defaultRelayState": "",
            "destination": "http://destination.okta.com",
            "digestAlgorithm": "SHA256",
            "honorForceAuthn": True,
            "idpIssuer": "http://www.okta.com/${org.externalKey}",
            "inlineHooks": [],
            "recipient": "http://recipient.okta.com",
            "requestCompressed": False,
            "responseSigned": True,
            "signatureAlgorithm": "RSA_SHA256",
            "slo": {"enabled": False},
            "ssoAcsUrl": "http://example.okta.com",
            "subjectNameIdFormat": "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress",
            "subjectNameIdTemplate": "${user.userName}",
        },
    },
    "signOnMode": ApplicationSignOnMode('SAML_2_0'),
    "status": "ACTIVE",
    "visibility": {
        "appLinks": {"testOrgCustomSamlAppLink": True},
        "autoSubmitToolbar": False,
        "hide": {"iOS": False, "web": False},
    },
}


class MockResponseObj:
    def __init__(self, response):
        self.response = response

    def get_body(self):
        return self.response


class MockRequestExecutor(RequestExecutor):
    async def execute(self, *args, **kwargs):
        return self.response, None

    def set_response(self, response):
        self.response = MockResponseObj(response)


def test_known_sign_on_mode():
    response = copy.deepcopy(SAML_APP_RESP_DICT)

    config = {
        "orgUrl": "https://test_org.okta.com",
        "token": "test_token",
        "requestExecutor": MockRequestExecutor,
    }
    client = Client(config)

    # check list applications
    client._request_executor.set_response([response])
    event_loop = asyncio.get_event_loop()
    result, resp, err = event_loop.run_until_complete(client.list_applications())
    assert type(result[0]) == SamlApplication
    assert result[0].as_dict() == EXPECTED_SAML_APP_AS_DICT

    # check get application
    client._request_executor.set_response(response)
    event_loop = asyncio.get_event_loop()
    result, resp, err = event_loop.run_until_complete(client.get_application("test_id"))
    assert type(result) == SamlApplication
    assert result.as_dict() == EXPECTED_SAML_APP_AS_DICT


def test_unknown_sign_on_mode():
    response = copy.deepcopy(SAML_APP_RESP_DICT)
    response["signOnMode"] = "UNKNOWN_SIGN_ON_MODE"
    expected = copy.deepcopy(EXPECTED_SAML_APP_AS_DICT)
    expected["signOnMode"] = ApplicationSignOnMode("UNKNOWN_SIGN_ON_MODE")
    expected["settings"] = {
        "app": {},
        "notifications": {
            "vpn": {"network": {"connection": "DISABLED", "exclude": [], "include": []}}
        },
    }

    config = {
        "orgUrl": "https://test_org.okta.com",
        "token": "test_token",
        "requestExecutor": MockRequestExecutor,
    }
    client = Client(config)

    # check list applications
    client._request_executor.set_response([response])
    event_loop = asyncio.get_event_loop()
    result, resp, err = event_loop.run_until_complete(client.list_applications())
    # verify if result fallbacks to generic Application
    assert type(result[0]) != SamlApplication
    assert type(result[0]) == Application
    assert result[0].as_dict() == expected

    # check get application
    client._request_executor.set_response(response)
    event_loop = asyncio.get_event_loop()
    result, resp, err = event_loop.run_until_complete(client.get_application("test_id"))
    # verify if result fallbacks to generic Application
    assert type(result) != SamlApplication
    assert type(result) == Application
    assert result.as_dict() == expected


def test_no_sign_on_mode():
    response = copy.deepcopy(SAML_APP_RESP_DICT)
    response["signOnMode"] = None
    expected = copy.deepcopy(EXPECTED_SAML_APP_AS_DICT)
    expected.pop("signOnMode")
    expected["settings"] = {
        "app": {},
        "notifications": {
            "vpn": {"network": {"connection": "DISABLED", "exclude": [], "include": []}}
        },
    }

    config = {
        "orgUrl": "https://test_org.okta.com",
        "token": "test_token",
        "requestExecutor": MockRequestExecutor,
    }
    client = Client(config)

    # check list applications
    client._request_executor.set_response([response])
    event_loop = asyncio.get_event_loop()
    result, resp, err = event_loop.run_until_complete(client.list_applications())
    assert type(result[0]) != SamlApplication
    assert type(result[0]) == Application
    assert result[0].as_dict() == expected

    # check get application
    client._request_executor.set_response(response)
    event_loop = asyncio.get_event_loop()
    result, resp, err = event_loop.run_until_complete(client.get_application("test_id"))
    assert type(result) != SamlApplication
    assert type(result) == Application
    assert result.as_dict() == expected



import copy
import pytest
import random
from string import ascii_lowercase
from tests.mocks import MockOktaClient
import okta.models as models


ADD_CUSTOM_ATTRIBUTE_DEFINITION = {
    "custom": {
         "id": "#custom",
         "type": "object",
         "properties": {
             "required": []
         }
    }
}

REMOVE_CUSTOM_ATTRIBUTE_DEFINITION = {
    "custom": {
         "id": "#custom",
         "type": "object",
         "properties": {}
    }
}

CUSTOM_ATTRIBUTE_PROPERTIES = {
     "title": "Test Custom Attribute",
     "description": "Custom attribute for testing purposes",
     "type": "string",
     "required": False,
     "minLength": 1,
     "maxLength": 20,
     "permissions": [
         {
             "principal": "SELF",
             "action": "READ_WRITE"
         }
     ]
}


class TestGroupSchemaResource:
    """
    Integration Tests for the Group Schema Resource
    """

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_group_schema(self, fs):
        client = MockOktaClient(fs)
        resp, _, err = await client.get_group_schema()
        assert err is None
        assert isinstance(resp, models.GroupSchema)
        assert isinstance(resp.definitions, models.GroupSchemaDefinitions)
        assert isinstance(resp.definitions.base, models.GroupSchemaBase)
        assert isinstance(resp.definitions.base.properties, models.GroupSchemaBaseProperties)
        assert resp.schema == 'http://json-schema.org/draft-04/schema#'

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_add_and_remove_custom_attribute(self, fs):
        client = MockOktaClient(fs)

        # make random custom attribute name, because it takes some time for cleanup
        random.seed(2021)
        custom_attribute_name = ''.join([ascii_lowercase[random.randint(0, 25)] for i in range(20)])
        definition = copy.deepcopy(ADD_CUSTOM_ATTRIBUTE_DEFINITION)
        definition['custom']['properties'][custom_attribute_name] = CUSTOM_ATTRIBUTE_PROPERTIES

        try:
            resp, _, err = await client.update_group_schema({'definitions': definition})
            assert err is None
            assert custom_attribute_name in resp.definitions.custom.properties

        finally:
            definition = copy.deepcopy(REMOVE_CUSTOM_ATTRIBUTE_DEFINITION)
            definition['custom']['properties'][custom_attribute_name] = None
            resp, _, err = await client.update_group_schema({'definitions': definition}, keep_empty_params=True)
            assert err is None
            assert custom_attribute_name not in resp.definitions.custom.properties

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_custom_attribute(self, fs):
        client = MockOktaClient(fs)

        # make random custom attribute name, because it takes some time for cleanup
        random.seed(2021)
        custom_attribute_name = ''.join([ascii_lowercase[random.randint(0, 25)] for i in range(20)])
        definition = copy.deepcopy(ADD_CUSTOM_ATTRIBUTE_DEFINITION)
        definition['custom']['properties'][custom_attribute_name] = CUSTOM_ATTRIBUTE_PROPERTIES

        try:
            resp, _, err = await client.update_group_schema({'definitions': definition})
            assert err is None
            assert custom_attribute_name in resp.definitions.custom.properties
            assert resp.definitions.custom.properties[custom_attribute_name]['title'] == 'Test Custom Attribute'

            # update custom attribute
            resp.definitions.custom.properties[custom_attribute_name]['title'] = 'New Title'
            resp, _, err = await client.update_group_schema(resp)
            assert err is None
            assert resp.definitions.custom.properties[custom_attribute_name]['title'] == 'New Title'

        finally:
            definition = copy.deepcopy(REMOVE_CUSTOM_ATTRIBUTE_DEFINITION)
            definition['custom']['properties'][custom_attribute_name] = None
            resp, _, err = await client.update_group_schema({'definitions': definition}, keep_empty_params=True)
            assert err is None
            assert custom_attribute_name not in resp.definitions.custom.properties



import pytest
from tests.mocks import MockOktaClient
from okta.models import ThreatInsightConfiguration


class TestThreatInsightConfiguration:
    """
    Integration Tests for the Threat Insight Resource
    """

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_threat_insight_configuration(self, fs):
        client = MockOktaClient(fs)
        resp, _, err = await client.get_current_configuration()
        assert isinstance(resp, ThreatInsightConfiguration)
        assert '_links' in resp.as_dict().keys()
        assert 'action' in resp.as_dict().keys()
        assert 'excludeZones' in resp.as_dict().keys()

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_threat_insight_configuration(self, fs):
        client = MockOktaClient(fs)
        old_config, _, err = await client.get_current_configuration()
        new_config = old_config
        new_config.action = 'audit'
        try:
            updated_config, _, err = await client.update_configuration(new_config)
            assert updated_config.action == 'audit'
        finally:
            _, _, err = await client.update_configuration(old_config)
            assert err is None



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from urllib.parse import urlencode
from okta.models.policy\
    import Policy
from okta.models.policy_rule\
    import PolicyRule
from okta.utils import format_url
from okta.api_client import APIClient
from okta.constants import find_policy_model
from okta.constants import find_policy_rule_model


class PolicyClient(APIClient):
    """
    A Client object for the Policy resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_policies(
            self, query_params={},
            keep_empty_params=False
    ):
        """
        Gets all policies with the specified type.
        Args:
            query_params {dict}: Map of query parameters for request
            [query_params.type] {str}
            [query_params.status] {str}
            [query_params.expand] {str}
        Returns:
            list: Collection of Policy instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Policy)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(
                    find_policy_model(item["type"])(
                        self.form_response_body(item)
                        )
                    )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_policy(
            self, policy, query_params={},
            keep_empty_params=False
    ):
        """
        Creates a policy.
        Args:
            {policy}
            query_params {dict}: Map of query parameters for request
            [query_params.activate] {str}
        Returns:
            Policy
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        if isinstance(policy, dict):
            body = policy
        else:
            body = policy.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Policy)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_policy(
            self, policyId,
            keep_empty_params=False
    ):
        """
        Removes a policy.
        Args:
            policy_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_policy(
            self, policyId, query_params={},
            keep_empty_params=False
    ):
        """
        Gets a policy.
        Args:
            policy_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.expand] {str}
        Returns:
            Policy
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Policy)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_policy(
            self, policyId, policy,
            keep_empty_params=False
    ):
        """
        Updates a policy.
        Args:
            policy_id {str}
            {policy}
        Returns:
            Policy
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}
            """)

        if isinstance(policy, dict):
            body = policy
        else:
            body = policy.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Policy)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def activate_policy(
            self, policyId,
            keep_empty_params=False
    ):
        """
        Activates a policy.
        Args:
            policy_id {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/lifecycle/activate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def deactivate_policy(
            self, policyId,
            keep_empty_params=False
    ):
        """
        Deactivates a policy.
        Args:
            policy_id {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/lifecycle/deactivate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def list_policy_rules(
            self, policyId,
            keep_empty_params=False
    ):
        """
        Enumerates all policy rules.
        Args:
            policy_id {str}
        Returns:
            list: Collection of PolicyRule instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/rules
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, PolicyRule)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(
                    find_policy_rule_model(item["type"])(
                        self.form_response_body(item)
                        )
                    )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_policy_rule(
            self, policyId, policy_rule,
            keep_empty_params=False
    ):
        """
        Creates a policy rule.
        Args:
            policy_id {str}
            {policy_rule}
        Returns:
            PolicyRule
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/rules
            """)

        if isinstance(policy_rule, dict):
            body = policy_rule
        else:
            body = policy_rule.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, PolicyRule)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_rule_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_policy_rule(
            self, policyId, ruleId,
            keep_empty_params=False
    ):
        """
        Removes a policy rule.
        Args:
            policy_id {str}
            rule_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/rules/{ruleId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_policy_rule(
            self, policyId, ruleId,
            keep_empty_params=False
    ):
        """
        Gets a policy rule.
        Args:
            policy_id {str}
            rule_id {str}
        Returns:
            PolicyRule
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/rules/{ruleId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, PolicyRule)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_rule_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_policy_rule(
            self, policyId, ruleId, policy_rule,
            keep_empty_params=False
    ):
        """
        Updates a policy rule.
        Args:
            policy_id {str}
            rule_id {str}
            {policy_rule}
        Returns:
            PolicyRule
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/rules/{ruleId}
            """)

        if isinstance(policy_rule, dict):
            body = policy_rule
        else:
            body = policy_rule.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, PolicyRule)

        if error:
            return (None, response, error)

        try:
            body = self.form_response_body(response.get_body())
            result = find_policy_rule_model(body["type"])(body)
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def activate_policy_rule(
            self, policyId, ruleId,
            keep_empty_params=False
    ):
        """
        Activates a policy rule.
        Args:
            policy_id {str}
            rule_id {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/rules/{ruleId}
                /lifecycle/activate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def deactivate_policy_rule(
            self, policyId, ruleId,
            keep_empty_params=False
    ):
        """
        Deactivates a policy rule.
        Args:
            policy_id {str}
            rule_id {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/policies/{policyId}/rules/{ruleId}
                /lifecycle/deactivate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.models.group_schema\
    import GroupSchema
from okta.utils import format_url
from okta.api_client import APIClient


class GroupSchemaClient(APIClient):
    """
    A Client object for the GroupSchema resource.
    """

    def __init__(self):
        self._base_url = ""

    async def get_group_schema(
            self,
            keep_empty_params=False
    ):
        """
        Fetches the group schema
        Args:
        Returns:
            GroupSchema
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/group/default
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, GroupSchema)

        if error:
            return (None, response, error)

        try:
            result = GroupSchema(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_group_schema(
            self, group_schema,
            keep_empty_params=False
    ):
        """
        Updates, adds ore removes one or more custom Group Prof
        ile properties in the schema
        Args:
            {group_schema}
        Returns:
            GroupSchema
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/group/default
            """)

        if isinstance(group_schema, dict):
            body = group_schema
        else:
            body = group_schema.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, GroupSchema)

        if error:
            return (None, response, error)

        try:
            result = GroupSchema(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)
